﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using ICSharpCode.NRefactory.CSharp;
using ICSharpCode.NRefactory.CSharp.Completion;
using ICSharpCode.NRefactory.CSharp.Resolver;
using ICSharpCode.NRefactory.CSharp.TypeSystem;
using ICSharpCode.NRefactory.Editor;
using ICSharpCode.NRefactory.TypeSystem;

namespace HeuristicLab.CodeEditor {
  internal class CSharpCodeCompletionContext {
    public readonly IDocument Document;
    public readonly int Offset;
    public readonly IProjectContent ProjectContent;
    public readonly ICompilation Compilation;
    public readonly CSharpResolver Resolver;
    public readonly CSharpTypeResolveContext TypeResolveContextAtCaret;
    public readonly ICompletionContextProvider CompletionContextProvider;

    public CSharpCodeCompletionContext(IDocument document, int offset, IProjectContent projectContent) {
      Document = new ReadOnlyDocument(document, document.FileName);
      Offset = offset;

      var parser = new CSharpParser();
      var syntaxTree = parser.Parse(document, document.FileName);
      syntaxTree.Freeze();
      var unresolvedFile = syntaxTree.ToTypeSystem();

      ProjectContent = projectContent.AddOrUpdateFiles(unresolvedFile);
      Compilation = projectContent.CreateCompilation();

      var location = document.GetLocation(offset);
      Resolver = unresolvedFile.GetResolver(Compilation, location);
      TypeResolveContextAtCaret = unresolvedFile.GetTypeResolveContext(Compilation, location);
      CompletionContextProvider = new DefaultCompletionContextProvider(document, unresolvedFile);
    }
  }
}
